@@ -4,9 +4,11 @@ MYLLM Main 🤖
 
 """
 
+import base64
 import importlib
 
 from loguru import logger
+from playwright.async_api import async_playwright
 
 from myllm import __version__
 from myllm.config import settings
@@ -27,6 +29,7 @@ class MyLLM:
         get_chats(self, prompt)
         export_chat_history(self)
         clear_chat_history(self)
+        import_chat_history(self)
 
     """
 
@@ -57,46 +60,51 @@ class MyLLM:
             None
         """
         # Check if the module is enabled
-        self.enabled = settings.myllm_enabled or True
+        self.enabled = settings.myllm_enabled
 
         # Set the prefix for AI agents
         self.ai_agent_mode = settings.ai_agent_mode or False
         self.ai_agent_prefix = settings.ai_agent_prefix or ""
         self.ai_agent_suffix = settings.ai_agent_suffix or ""
 
+        # Set the browser settings
+        self.browser_url = settings.browser_url or "https://google.com"
+        self.browser_headless = settings.browser_headless or True
+
         # Create a mapping of library names to client classes
         self.client_classes = self.get_all_client_classes()
         # logger.debug("client_classes available {}", self.client_classes)
 
         if not self.enabled:
-            logger.info("Module is disabled. No clients will be created.")
+            logger.info("Module is disabled. No Client will be created.")
             return
         self.clients = []
 
         # Create a client for each client in settings.myllm
         for name, client_config in settings.myllm.items():
-            # Skip template and empty string client names
-            if name in ["", "template"] or not client_config.get("enabled"):
+            if (
+                # Skip empty client configs
+                client_config is None
+                # Skip non-dict client configs
+                or not isinstance(client_config, dict)
+                # Skip template and empty string client names
+                or name in ["", "template"]
+                # Skip disabled clients
+                or not client_config.get("enabled")
+            ):
                 continue
-            try:
                 # Create the client
-                client = self._create_client(**client_config, name=name)
-                # If the client has a valid client attribute, append it to the list
-                if client and getattr(client, "client", None):
-                    self.clients.append(client)
-            except Exception as e:
-                # Log the error if the client fails to be created
-                logger.error(f"Failed to create client {name}: {e}")
+            logger.debug("Creating client {}", name)
+            client = self._create_client(**client_config, name=name)
+            # If the client has a valid client attribute, append it to the list
+            if client and getattr(client, "client", None):
+                self.clients.append(client)
 
         # Log the number of clients that were created
         logger.info(f"Loaded {len(self.clients)} clients")
         if not self.clients:
             logger.warning(
-                """
-                No clients were created.
-                Check your settings or disable the module.
-                https://talky.readthedocs.io/en/latest/02_config.html
-                """
+                "No Client were created. Check your settings or disable the module."
             )
 
     def _create_client(self, **kwargs):
@@ -132,14 +140,10 @@ class MyLLM:
             library is not supported.
 
         """
-        library = kwargs.get("llm_library") or kwargs.get("library") or "notset"
-        client_class = self.client_classes.get(f"{library.capitalize()}Handler")
-
-        if client_class is None:
-            logger.warning(f"library {library} not supported")
-            return None
-
-        return client_class(**kwargs)
+        library = kwargs.get("library") or kwargs.get("llm_library") or "g4f"
+        return self.client_classes.get(f"{library.capitalize()}Handler", None).__call__(
+            **kwargs
+        )
 
     def get_all_client_classes(self):
         """
@@ -187,7 +191,14 @@ class MyLLM:
         """
 
         _chats = [
-            f"{self.ai_agent_prefix} {client.name}\n{data} {self.ai_agent_suffix}"
+            (
+                data
+                if len(self.clients) == 1 and not self.ai_agent_mode
+                else (
+                    f"{self.ai_agent_prefix} {client.name}\n"
+                    f"{data} {self.ai_agent_suffix}"
+                )
+            )
             for client in self.clients
             if (data := await client.chat(prompt)) is not None and data.strip()
         ]
@@ -195,6 +206,45 @@ class MyLLM:
         if _chats:
             return "\n".join(_chats)
 
+    async def vision(self, prompt=None):
+        """
+        Asynchronously processes a base64-encoded image
+        and returns the responses from each client.
+
+        Parameters:
+        base64_image (str): A base64-encoded image.
+
+        Returns:
+        str: A string containing the responses
+        from each client, separated by newlines.
+
+        """
+        _chats = []
+        for client in self.clients:
+            try:
+                if prompt is None:
+                    data = await client.vision()
+                else:
+                    data = await client.vision(prompt=prompt)
+                if data is not None and data.strip():
+                    if len(self.clients) == 1 and not self.ai_agent_mode:
+                        _chats.append(data)
+                    else:
+                        _chats.append(
+                            f"{self.ai_agent_prefix} {client.name}\n"
+                            f"{data} {self.ai_agent_suffix}"
+                        )
+            except AttributeError as e:
+                logger.error(
+                    f"Error processing with {client.name}: {e}. "
+                    "Make sure the client has a vision() method."
+                )
+            except Exception as e:
+                logger.error(f"Error processing with {client.name}: {e}")
+
+        if _chats:
+            return "\n".join(_chats)
+
     async def export_chat_history(self):
         """
         Asynchronous function to export chat history for each
@@ -211,3 +261,54 @@ class MyLLM:
         """
         for client in self.clients:
             await client.clear_chat_history()
+
+    async def import_chat_history(self):
+        """
+        Asynchronously clears the chat history for each
+        client in the list of clients.
+        """
+        for client in self.clients:
+            await client.import_chat_history()
+
+    async def browse_url(self, url: str = "https://google.com") -> None:
+        """
+        Browse URL and save a screenshot of the page.
+        """
+        logger.info("Browsing URL: {}", url)
+        if url is None:
+            url = self.browser_url
+        async with async_playwright() as playwright:
+            browser = await playwright.chromium.launch(headless=self.browser_headless)
+            await browser.new_context(
+                locale="en-US",
+                timezone_id="America/Los_Angeles",
+                device_scale_factor=1,
+                screen={"width": 1920, "height": 1080},
+                extra_http_headers={
+                    "Accept": (
+                        "text/html,application/xhtml+xml,"
+                        "application/xml;q=0.9,*/*;q=0.8"
+                    ),
+                    "Accept-Encoding": "gzip, deflate, br",
+                    "Accept-Language": "en-GB,en;q=0.9",
+                    "Cache-Control": "max-age=0",
+                    "Connection": "keep-alive",
+                    "Sec-Fetch-Dest": "document",
+                    "Sec-Fetch-Mode": "navigate",
+                    "Sec-Fetch-Site": "same-origin",
+                    "User-Agent": (
+                        "Mozilla/5.0 "
+                        "(Macintosh; Intel Mac OS X 10_15_7) "
+                        "AppleWebKit/605.1.15 (KHTML, like Gecko) "
+                        "Version/17.2.1 Safari/605.1.15"
+                    ),
+                },
+            )
+            page = await browser.new_page()
+            await page.goto(url)
+            # await asyncio.sleep(2)
+            # await page.mouse.wheel(0, 300)
+            screenshot_bytes = await page.screenshot()
+            base64_image = base64.b64encode(screenshot_bytes).decode("utf-8")
+            await browser.close()
+        return await self.vision(base64_image=base64_image)
